!  SUBROUTINES:
!	NactiKonstanty     - Nate konstanty ze souboru KonstantyNPT.ini
!	NactiSouradnice	   - Nate souadnice atom ze souboru xyz.ini

!  FUNCTIONS:
!	VypoctiPolomerKoule     - Vpoet polomru koule z jejho objemu
!	VypoctiPolomerKrychle     - Vpoet polomru krychle z jejho objemu
!

!****************************************************************************************************************************
!
!  MODULE: Inicializace
!
!  PURPOSE: Naten a nastaven promnnch
!
!****************************************************************************************************************************


module Inicializace
implicit none

	integer i, j, i1, i2, krok, iT, iP				! Promnn cyklu
	double precision teplotaMD, entalpieMD	! Hodnoty pro MolDraw
	double precision, parameter :: konstKoule = 0.620350491 ! 3/(4*pi) ** 1/3 - Pro vpoet polomru z objemu koule	
	double precision, parameter :: konstKrychle = 0.5 ! 1/2
	 
	double precision random
	
	! Konstanty ze souboru KonstantyNPT.ini

	! Vbr
	integer tvar	 ! Tvar kontejneru: | 0 - koule | 1 - krychle
	integer pocitat  ! Potat: 0 - Struktury | 1 - Termodynamiku
	integer jednotky ! Simulace:  0 - bezrozmrn | 1 - v jednotkch
	
	! Obecn
	integer n, maxKroku, upravDelty ! Poet atom | Poet opakovn MC cyklu | Kalibrace posunu a objemu
	double precision pravdepodobnost ! Pravdpodobnost vbru posunu atom (s opanou pravdpodobnost se zmn objem)
	
	! Struktury
	integer upravMD, upravT ! Poet krok po kolika: do MolDraw | Snen teploty 	
	double precision  dT	! Koeficient zmny teploty

	! Termodynamika
	integer rozsahP, rozsahT, dP, ekvilibrizace ! Rozsah tlak a teplot | Zmna tlaku | Doba ekvilibrizan fze
	integer vypoctiTermodyn ! Po kolika krocch provdt termodynamick vpoty

	double precision T, Tkoncova, P, V	! Teplota - poten, koncov | Tlak | Objem
	double precision k, sigma, epsilon	! Boltzmanova konstanta | sigma | epsilon	
	double precision dZeta, dV	! Koeficienty zmny: zt | objemu
	double precision  r, eV, maxV	! Polomr | 1 eV | Hranice objemu
		
		
	! Souadnice: atom | Zt | Novch poloh 
	double precision, dimension (:), allocatable :: Rx, Ry, Rz, ZetaX, ZetaY, ZetaZ, xN, yN, zN

contains

! ***************************************************************************************************************************
	
	! Nate konstanty ze souboru KonstantyNpT.ini do promnnch
	subroutine NactiKonstanty()
		
		open (10, action="read", file="KonstantyNpT.ini")
			read (10, *) pocitat
			read (10, *) tvar			
			read (10, *) jednotky
			read (10, *) ! -----------------------------------------------
			read (10, *) n
			read (10, *) pravdepodobnost
			read (10, *) upravDelty
			read (10, *) maxKroku			
			read (10, *) ! ----------------- STRUKTURY -------------------		
			read (10, *) upravT			
			read (10, *) upravMD
			read (10, *) dT			
			read (10, *) ! --------------- TERMODYNAMIKA -----------------
			read (10, *) rozsahP
			read (10, *) rozsahT
			read (10, *) dP
			read (10, *) ekvilibrizace
			read (10, *) vypoctiTermodyn
			read (10, *) ! Simulace bezrozmrn
			read (10, *) k
			read (10, *) sigma 
			read (10, *) epsilon
			read (10, *) dZeta
			read (10, *) dV			
			read (10, *) T
			read (10, *) Tkoncova
			read (10, *) P
			read (10, *) V
			read (10, *) maxV
			read (10, *) eV
			if (jednotky == 1) then
				read (10, *)  ! Simulace s jednotkami
				read (10, *) k
				read (10, *) sigma 
				read (10, *) epsilon
				read (10, *) dZeta
				read (10, *) dV			
				read (10, *) T
				read (10, *) Tkoncova
				read (10, *) P
				read (10, *) V
				read (10, *) maxV
				read (10, *) eV														
			end if
			
		close (10)
				
	end subroutine
	
! ***************************************************************************************************************************

	! Naten souadnic zt ze souboru xyz.ini do pol
	subroutine NactiSouradnice()

		allocate (Rx(1:n))
		allocate (Ry(1:n))
		allocate (Rz(1:n))
		allocate (xN(1:n))
		allocate (yN(1:n))
		allocate (zN(1:n))
		allocate (ZetaX(1:n))
		allocate (ZetaY(1:n))
		allocate (ZetaZ(1:n))
		
		if (tvar == 0) then
			r = vypoctiPolomerKoule(V)
		elseif (tvar == 1) then
			r = vypoctiPolomerKrychle(V)
		end if	
	
		open(20,action="read", file="xyz.ini")

		read(20,*)	
				
		do i = 1,n
			read(20,*) Rx(i), Ry(i), Rz(i)
			ZetaX(i) = Rx(i) / r
			ZetaY(i) = Ry(i) / r
			ZetaZ(i) = Rz(i) / r
		end do		

		close(20)	

	end subroutine

! ***************************************************************************************************************************
	
	! Vpoet polomru koule z jejho objemu
	double precision function VypoctiPolomerKoule(objem)
		double precision, intent(in) :: objem		
		!	(3/4pi * V) ** 1/3
		vypoctiPolomerKoule = konstKoule * (objem**0.333333334)

	end function

! ***************************************************************************************************************************

	! Vpoet polomru krychle z jejho objemu
	double precision function VypoctiPolomerKrychle(objem)
		double precision, intent(in) :: objem
		
		!	sqrt(3) / 2 * V ** 1/3
		vypoctiPolomerKrychle = konstKrychle * (objem**0.333333334)

	end function

! ***************************************************************************************************************************

end module Inicializace