!  SUBROUTINES:
!	NaplnEnergie     - Vypoten poten energie (i entalpie) klstru
!	PosunAtomu		 - Posun hodnot Zeta a test pijet
!	ZmenaObjemu		 - Zmna objemu, pepotn hodnot atom a test pijet
!	UpravPosun		 - Zvt (resp. zmen) posun zt, pokud je mnoho konfigurac pijmno (resp. zamtno)	
!	UpravObjem		 - Zvt (resp. zmen) velikost objemu, pokud je mnoho konfigurac pijmno (resp. zamtno)				 

!******************************************************************************************************************
!
!  MODULE: MonteCarlo
!
!  PURPOSE: Proveden samotn simulace Monte Carlo: Posun atom a zmna objemu
!
!******************************************************************************************************************

module MonteCarlo
use Potencial
implicit none
	
	double precision prijatoV, zamitnutoV, prijatoA, zamitnutoA ! Poet pijatch (resp. zamtnutch) atom a objem
	double precision, dimension (:,:), allocatable :: energie, energieZkus ! Energie provch dvojic
	double precision xT, yT, zT ! Souadnice tit
	double precision Ustara, Unova, Hstara, Hnova ! Energie a entalpie klstru
	double precision Vn, rN ! Nov objem a polomr	
	logical preskocit	! v ppad, e Zty pekro stny jednotkov koule - > zamtnut a peskoen cyklu	

contains

!******************************************************************************************************************
	
	subroutine NaplnEnergie()
		
		allocate (energie(1:n,1:n))
		allocate (energieZkus(1:n,1:n))			

		! Vypoten energie pro vechny prov dvojice
		do i = 1,n
		    do j = i+1,n
				energie (i,j) = LJpotencial(Rx(i), Ry(i), Rz(i),Rx(j), Ry(j), Rz(j))
				energie (j,i) = energie(i,j)			
				Ustara = Ustara + energie(i,j)						  
			end do
		end do
		
		! Spoten entalpie
		if (jednotky == 1) then
			Hstara = Ustara + P * V * 10E-27
		else
			Hstara = Ustara + P * V
		end if
			
	end subroutine NaplnEnergie

!******************************************************************************************************************

	subroutine PosunAtomu()		

		do i = 1,n

			! Uchovn starch souadnic zt pro ppad zamtnut
			xN = ZetaX
			yN = ZetaY
			zN = ZetaZ

			! Nhodn posun jedn zty
			call random_number(random)
			xN(i) = ZetaX(i) + (random-0.5) * dZeta			
			call random_number(random)
			yN(i) = ZetaY(i) + (random-0.5) * dZeta		
			call random_number(random)
			zN(i) = ZetaZ(i) + (random-0.5) * dZeta			
		
			! Kontrola, zda je zta v jednotkov			
			
			if (tvar == 0) then
				! kouli
				if (sqrt(xN(i)**2 + yN(i)**2 + zN(i)**2)>=1) then
					zamitnutoA = zamitnutoA + 1									
					preskocit = .TRUE.
					exit
				end if			
			elseif (tvar == 1) then
				! krychli
				if ( ( abs (xN(i)) >=1 ) .OR. ( abs (yN(i)) >=1 ) .OR. ( abs (zN(i)) >=1 ) ) then
					zamitnutoA = zamitnutoA + 1									
					preskocit = .TRUE.
					exit
				end if	
			end if						
			
			! Pokud ne, konfiguraci zamtneme a peskome cyklus na dal ztu
			if (preskocit == .TRUE.) then							
				preskocit = .FALSE.
				cycle
			end if

			! Vypoten souadnic atom z hodnot zt
			Rx(i) = xN(i) * r
			Ry(i) = yN(i) * r
			Rz(i) =	zN(i) * r	
			
			! Hodnoty pro MolDraw	
			entalpieMD = Hstara/eV
			teplotaMD = T
								 
			energieZkus = energie	
			
			! Spoten energie vech provch dvojic				 
			do j = 1,n							
				if (i/=j) then
					energieZkus (i,j) = LJpotencial(Rx(i), Ry(i), Rz(i),Rx(j), Ry(j), Rz(j)) 
					energieZkus (j,i) = energieZkus (i, j)							
				end if
			end do
					
			Unova = 0
			
			! Spoten nov energie jako souet energi pr
			do i2 = 1,n
				do j = i2+1,n							
					Unova = Unova + energieZkus (i2,j)
				end do
			end do	
			
			! Spoten nov entalpie			

			if (jednotky == 1) then
				! pokud potme v jednotkch, pevedeme objem na metry
				Hnova = Unova + P * V * 10E-27
			else
				Hnova = Unova + P * V
			end if
			
			! Metropolisv test	
			call random_number(random)
			
			if (random < exp ( -(Hnova-Hstara)/(k*T) )) then			
				
				! Konfigurace pijata
				prijatoA = prijatoA + 1	

				ZetaX = xN
				ZetaY = yN
				ZetaZ = zN	
								
				energie = energieZkus				
				Ustara = Unova			
				Hstara = Hnova
			

			else 							
				! Konfigurace zamtnuta, vracme hodnoty zpt
				zamitnutoA = zamitnutoA + 1

				Rx(i) = ZetaX(i) * r
				Ry(i) = ZetaY(i) * r
				Rz(i) =	ZetaZ(i) * r
								
			end if
					
		end do

	end subroutine PosunAtomu

!******************************************************************************************************************

	subroutine ZmenaObjemu()

		! Nhodn zmna objemu
		call random_number (random)
		Vn = V + (random - 0.5) * dV
		
		! Vypotn novho polomru z objemu
		if (tvar == 0) then
			rN = vypoctiPolomerKoule(Vn)
		elseif (tvar == 1) then
			rN = vypoctiPolomerKrychle(Vn)
		end if

		! Pepotn hodnot atom
		do i = 1,n
			xN(i) = ZetaX(i) * rN
			yN(i) = ZetaY(i) * rN
			zN(i) = ZetaZ(i) * rN			
		end do	
		
		! Hodnoty pro MolDraw		
		entalpieMD = Hstara/eV
		teplotaMD = T

		Unova = 0
	
		! Spoten energie vech provch dvojic	
		do i=1,n				 
			do j = 1,n							
				if (i/=j) then
					energieZkus (i,j) = LJpotencial(xN(i), yN(i), zN(i),xN(j), yN(j), zN(j))
					Unova = Unova + energieZkus (i,j)																						
				end if
			end do
		end do

		Unova = Unova/2

		! Spoten nov entalpie
		if (jednotky == 1) then
			! pokud potme v jednotkch, pevedeme objem na metry
			Hnova = Unova + P * Vn * 10E-27
		else
			Hnova = Unova + P * Vn
		end if
				
		call random_number(random)	

		! Metropolisv test	a hranice objemu
		if ( (random < (Vn/V) ** n * exp ( -(Hnova-Hstara)/(k*T) )) .AND. (Vn<=maxV) ) then
			
			! Konfigurace pijata
			prijatoV = prijatoV + 1
				
			r = rN
			V = Vn
			Rx = xN
			Ry = yN
			Rz = zN

			energie = energieZkus
							
			Ustara = Unova
			Hstara = Hnova
			
		else 
			! Konfigurace zamtnuta
			zamitnutoV = zamitnutoV + 1				
		end if
		
	end subroutine ZmenaObjemu

!******************************************************************************************************************
	
	! Kalibrace posunu
	subroutine UpravPosun()
		! Pokud je pijmno moc atom - > zvtme posun, jinak jej zmenme
		if (prijatoA/(prijatoA+zamitnutoA) > 0.5) then 
			dZeta = dZeta * 1.05
		else
			dZeta = dZeta / 1.05
		end if	
					
		prijatoA = 0
		zamitnutoA	= 0	

	end subroutine UpravPosun

!******************************************************************************************************************

	! Kalibrace objemu
	subroutine UpravObjem()
		! pokud je pijmno moc atom - > zvtme objem, jinak jej zmenme
		if (prijatoV/(prijatoV+zamitnutoV) > 0.5) then 
			dV = dV * 1.05
		else
			dV = dV / 1.05
		end if

		prijatoV = 0
		zamitnutoV = 0

	end subroutine UpravObjem

!******************************************************************************************************************

end module MonteCarlo