!**********************************************************************************
!
!  NpT simulace atom klstr metodou Monte Carlo
!
!  Autor: David Pgmek
!

!**********************************************************************************
!
!  PROGRAM: MC_atomy_NPT
!
!  PURPOSE: Vstupn bod programu - Posun a zmna objemu, sniovn teploty, prava posunu/objemu, vpis do MolDraw
!
!*****************************************************************************************************************

program MC_atomy_NpT

use MonteCarlo
use MolDraw
use Termodynamika

implicit none
	
	call random_seed()		
	
	call NactiKonstanty()	  

	call NactiSouradnice()
	
	call NaplnEnergie()
	
	! Potat struktury nebo termodynamiku

	if (pocitat == 0) then
		! Potat struktury
		
		! Soubor s hodnotami pro MolDraw
		open (30,file="ar_mc.xyz",action="write")

		krok = 1

		do while ( ( (T > Tkoncova) .AND. (Tkoncova/=0) ) .OR. (krok <= maxKroku) )		
			
			call random_number(random)
			
			! Vybere Posun atomu s pravdpodobnost pravdepodobnost
			if (random < pravdepodobnost) then
				call PosunAtomu()
			else
				call ZmenaObjemu()
			end if		

			! Sn teplotu
			if  ( (upravT/=0) .AND. (mod(krok,upravT) == 0) ) then			
				T = T * dT
			end if 
			
			! Uprav posun a objem
			if  ( (upravDelty/=0) .AND. (mod(krok,upravDelty) == 0) ) then
					call UpravPosun()			
					call UpravObjem()
			end if		
			

			! Zape do MolDraw souadnice klstru, jeho teplotu a entalpii
			if ( (upravMD>0) .AND. (mod(krok,upravMD) == 0) ) then			
				call ZapisMolDraw(Rx,Ry,Rz, entalpieMD, teplotaMD, V)
			end if

			krok = krok + 1
		end do

		close(30)

	elseif (pocitat == 1) then
		! Potat termodynamiku

		! Soubor pro Termodynamiku
		open (40,file="Termodynamika.txt",action="write")
		write (40,fmt= '( 5X, ''T'', 10X, ''H'', 20X, ''c'', 20X , ''V'' )' )
		
		dT = (T - Tkoncova) / rozsahT

		do iP = 1,rozsahP

			do iT =1,rozsahT
				
				do krok = 1, maxKroku
					call random_number(random)
						
					! Vybere Posun atomu s pravdpodobnost pravdepodobnost
					if (random < pravdepodobnost) then
						call PosunAtomu()
					else
						call ZmenaObjemu()
					end if	
					
					! Dokud nesjedeme na hledanou teplotu, linern ji snme
					if((mod(krok,10)==0) .AND. (T>Tkoncova))then
						T = T - 0.001	
					end if		
					
					! Pokud u mme sprvnou teplotu, provedeme ekvilibrizan fzi
					
					if ( (krok == 1) .AND. (T<=Tkoncova) ) then
						i1 = 1
						do while (i1 <= ekvilibrizace)

							call random_number(random)

							! Vybere Posun atomu s pravdpodobnost pravdepodobnost
							if (random < pravdepodobnost) then
								call PosunAtomu()
							else
								call ZmenaObjemu()
							end if
						
							i1 = i1 + 1
						end do
					end if

					if((T<=Tkoncova) .AND. (mod(krok,vypoctiTermodyn)==0))then 
						
						call TermodynamickeVypocty(Hstara)

						! Uprav posun a objem
						if  ( (upravDelty/=0) .AND. (mod(krok,upravDelty) == 0) ) then
							call UpravPosun()			
							call UpravObjem()
						end if

					end if
							
				end do

				Tkoncova = Tkoncova + dT
				
				!Zven teploty				
				T = T + dT

			end do

		Tkoncova = Tkoncova - rozsahT * dT

		! Zven tlaku
		P = P + dP
		
		write(40,*) "-------------------------------------------------"
		
		end do
			
		close(40)
	end if

	!read(*,*)
	
	
	

	deallocate (Rx)
	deallocate (Ry)
	deallocate (Rz)
	deallocate (xN)
	deallocate (yN)
	deallocate (zN)
	deallocate (ZetaX)
	deallocate (ZetaY)
	deallocate (ZetaZ)
	deallocate (energie)
	deallocate (energieZkus)
	
 end program MC_atomy_NpT

