!  SUBROUTINES:
!	NaplnEnergie     - Vypoten poten energie klstru
!	MonteCarloCyklus - Jeden Monte Carlo cyklus - posun atom a test pijet
!	UpravPosun		 - Zvt (resp. zmen) posun atom, pokud jich je mnoho pijmno (resp. zamtno)				 
!	Teziste			 - Vpoet tit a posun klastru do tohoto tit

!*******************************************************************************************************
!
!  MODULE: MonteCarlo
!
!  PURPOSE: Proveden samotn simulace Monte Carlo
!
!*******************************************************************************************************

module MonteCarlo
use Potencial
implicit none
	
	double precision prijato, zamitnuto  !poet atom pijatch / zamtnutch
	double precision, dimension (:,:), allocatable :: energie, energieZkus ! energie provch dvojic
	double precision xN, yN, zN ! Nov souadnice 
	double precision xT, yT, zT ! Souadnice tit
	double precision Ustara, Unova ! Star a nov energie	

contains

!*******************************************************************************************************
	
	subroutine NaplnEnergie()
		
		allocate (energie(1:n,1:n))
		allocate (energieZkus(1:n,1:n))

		!spocitani energii vsech dvojic atomu
		do i = 1,n
		    do j = i+1,n
				energie (i,j) = LJpotencial(Rx(i), Ry(i), Rz(i),Rx(j), Ry(j), Rz(j))
				energie (j,i) = energie(i,j)			
				Ustara = Ustara + energie(i,j)			  
			end do
		end do				
		
	end subroutine NaplnEnergie

!*******************************************************************************************************

	subroutine MonteCarloCyklus ()		

		do i = 1,n
			! Nhodn posun jednoho atomu klstru
			call random_number(random)
			xN = Rx(i) + (random-0.5) * posun		
			call random_number(random)
			yN = Ry(i) + (random-0.5) * posun		
			call random_number(random)
			zN = Rz(i) + (random-0.5) * posun
			
			! Pevod na eV pokud simulace v jednotkch
						
			energieMD = Ustara /eV
			teplotaMD = T
							 
			energieZkus = energie	
							 
			do j = 1,n							
				if (i/=j) then
					energieZkus (i, j) = LJpotencial(xN, yN, zN, Rx(j), Ry(j), Rz(j)) 
					energieZkus (j, i) = energieZkus (i, j)							
				end if
			end do
					
			Unova = 0

			do i2 = 1,n
				do j = i2+1,n							
					Unova = Unova + energieZkus (i2,j)
				end do
			end do
					
			call random_number(random)
			! Metropolisv test pijet / zamtnut	

			if (random < exp (-(Unova-Ustara)/(k*T))) then			
				
				call Teziste(Rx,Ry,Rz)

				! Pokud dan atom z koule neutk -> pijmeme
				if (sqrt(xN**2 + yN**2 + zN**2) < polomer) then					
					
					prijato = prijato + 1
										
					Rx(i) = xN
					Ry(i) = yN
					Rz(i) = zN					
					
					energie = energieZkus

					Ustara = Unova

				else 
					! Zamtnuto: atomu pekroil stny koule
					zamitnuto = zamitnuto + 1
				end if

			else 
				! Zamtnuto Metropolisovm testem
				zamitnuto = zamitnuto + 1
			end if
						
		end do

	end subroutine MonteCarloCyklus

!*******************************************************************************************************
	
	! Kalibrace posunu
	subroutine UpravPosun()	
		
		! Pokud je pijmno moc atom - > zvtme posun, jinak jej zmenme
		if (prijato/(prijato+zamitnuto) > 0.5) then
			 
			posun = posun *  1.05
		else
			posun = posun / 1.05
		end if		

		prijato = 0
		zamitnuto = 0
		
	end subroutine UpravPosun

!*******************************************************************************************************

	! Vypoten tit a nsledn posun do nj
	subroutine Teziste(x,y,z)
		
		double precision, dimension (:), intent(inout) :: x, y, z		
				
		xT = 0
		yT = 0
		zT = 0
		
		do j=1,n
			xT=	xT + x(j)
			yT=	yT + y(j)
			zT=	zT + z(j)
		end do

		xT = xT/n	
		yT = yT/n	
		zT = zT/n
		
		do j=1,n
			x(j) = x(j) - xT
			y(j) = y(j) - yT
			z(j) = z(j) - zT
		end do	
		
	end subroutine Teziste

!*******************************************************************************************************

end module MonteCarlo