!*****************************************************************************************************************
!
!  NVT simulace atom klstru metodou Monte Carlo
!
!  Autor: David Pgmek
!

!*****************************************************************************************************************
!
!  PROGRAM: MC_atomy_NVT
!
!  PURPOSE: Vstupn bod programu - opakovn cyklu Monte Carlo, sniovn teploty, prava posunu, vpis do MolDraw
!
!*****************************************************************************************************************

program MC_atomy_NVT

use MonteCarlo
use MolDraw
use Termodynamika

implicit none
	
	call random_seed()	

	call NactiKonstanty()	  

	call NactiSouradnice()
	
	call NaplnEnergie()	
	
	if (pocitat == 0) then
		! Potat struktury

		! Soubor s hodnotami pro MolDraw
		open (30,file="ar_mc.xyz",action="write")

		krok = 1
		do while ( ( (T > Tkoncova) .AND. (Tkoncova/=0) ) .OR. (krok <= maxKroku) )

			call MonteCarloCyklus()
			
			if ( (upravaTeploty /=0) .AND. (mod(krok,upravaTeploty) == 0) ) then
				
				T = T * dT				
					
			end if 		

			if ( (upravaPosunu /=0) .AND. (mod(krok,upravaPosunu) == 0) ) then			
				
				call UpravPosun()				
					
			end if	
				
			if ( (vypisMD /=0) .AND. (mod(krok,vypisMD) == 0) ) then
				call ZapisMolDraw(Rx,Ry,Rz, energieMD, teplotaMD)		
			end if
		
			krok = krok + 1			
		end do

		close (30)

	else if (pocitat == 1) then
		! Potat termodynamiku

		! Soubor pro Termodynamiku
		open (40,file="Termodynamika.txt",action="write")
		write (40,fmt= '( 10X, ''T'', 10X, ''<E>'', 20X, ''c'')' )
		
		do iT =1,rozsahT

			do krok=1,maxKroku
				call MonteCarloCyklus()

				if( (iT == 1) .AND. (mod(krok,10) == 0) .AND. (T>Tkoncova) )then			
					T = T - 0.001								
				end if 
			
				if(mod(krok,vypoctiTermodyn)==0)then 

					call TermodynamickeVypocty(Ustara)
				end if

				if ( (upravaPosunu /=0) .AND. (mod(krok,upravaPosunu) == 0) ) then
					call UpravPosun()	
				end if
				
			end do	
				
			T = T + 1
			Tkoncova = Tkoncova + 1

		end do

		close (40)
	
	end if	

	
	

	deallocate (Rx)
	deallocate (Ry)
	deallocate (Rz)
	deallocate (energie)
	deallocate (energieZkus)	

	!read(*,*)
	
 end program MC_atomy_NVT

